@@ -1,21 +1,28 @@
-import json, cStringIO
-from libpathod import pathoc, test, version, pathod
+import json
+import cStringIO
+import re
+
+from libpathod import pathoc, test, version, pathod, language
 import tutils
 
+
 def test_response():
-    r = pathoc.Response("1.1", 200, "Message", {}, None)
+    r = pathoc.Response("1.1", 200, "Message", {}, None, None)
     assert repr(r)
 
 
 class _TestDaemon:
     ssloptions = pathod.SSLOptions()
+
     @classmethod
     def setUpAll(self):
         self.d = test.Daemon(
             ssl=self.ssl,
             ssloptions=self.ssloptions,
             staticdir=tutils.test_data.path("data"),
-            anchors=[("/anchor/.*", "202")]
+            anchors=[
+                (re.compile("/anchor/.*"), language.parse_response("202"))
+            ]
         )
 
     @classmethod
@@ -27,22 +34,55 @@ class _TestDaemon:
 
     def test_info(self):
         c = pathoc.Pathoc(
-            "127.0.0.1",
-            self.d.port,
+            ("127.0.0.1", self.d.port),
             ssl = self.ssl
         )
         c.connect()
         r = c.request("get:/api/info")
         assert tuple(json.loads(r.content)["version"]) == version.IVERSION
 
+    def tval(
+        self,
+        requests,
+        showreq=False,
+        showresp=False,
+        explain=False,
+        showssl=False,
+        hexdump=False,
+        timeout=None,
+        ignorecodes=None,
+        ignoretimeout=None
+    ):
+        c = pathoc.Pathoc(("127.0.0.1", self.d.port), ssl=self.ssl)
+        c.connect()
+        if timeout:
+            c.settimeout(timeout)
+        s = cStringIO.StringIO()
+        for i in requests:
+            r = language.parse_requests(i)[0]
+            if explain:
+                r = r.freeze({})
+            c.print_request(
+                r,
+                showreq = showreq,
+                showresp = showresp,
+                explain = explain,
+                showssl = showssl,
+                hexdump = hexdump,
+                ignorecodes = ignorecodes,
+                ignoretimeout = ignoretimeout,
+                fp = s
+            )
+        return s.getvalue()
+
 
 class TestDaemonSSL(_TestDaemon):
     ssl = True
     ssloptions = pathod.SSLOptions(request_client_cert=True)
+
     def test_sni(self):
         c = pathoc.Pathoc(
-            "127.0.0.1",
-            self.d.port,
+            ("127.0.0.1", self.d.port),
             ssl = True,
             sni = "foobar.com"
         )
@@ -52,10 +92,12 @@ class TestDaemonSSL(_TestDaemon):
         d = json.loads(r.content)
         assert d["log"][0]["request"]["sni"] == "foobar.com"
 
+    def test_showssl(self):
+        assert "certificate chain" in self.tval(["get:/p/200"], showssl=True)
+
     def test_clientcert(self):
         c = pathoc.Pathoc(
-            "127.0.0.1",
-            self.d.port,
+            ("127.0.0.1", self.d.port),
             ssl = True,
             clientcert = tutils.test_data.path("data/clientcert/client.pem")
         )
@@ -68,29 +110,14 @@ class TestDaemonSSL(_TestDaemon):
 
 class TestDaemon(_TestDaemon):
     ssl = False
-    def tval(self, requests, showreq=False, showresp=False, explain=False, hexdump=False, timeout=None, ignorecodes=None, ignoretimeout=None):
-        c = pathoc.Pathoc("127.0.0.1", self.d.port)
-        c.connect()
-        if timeout:
-            c.settimeout(timeout)
-        s = cStringIO.StringIO()
-        for i in requests:
-            c.print_request(
-                i,
-                showreq = showreq,
-                showresp = showresp,
-                explain = explain,
-                hexdump = hexdump,
-                ignorecodes = ignorecodes,
-                ignoretimeout = ignoretimeout,
-                fp = s
-            )
-        return s.getvalue()
 
     def test_ssl_error(self):
-        c = pathoc.Pathoc("127.0.0.1", self.d.port, ssl = True)
+        c = pathoc.Pathoc(("127.0.0.1", self.d.port), ssl = True)
         tutils.raises("ssl handshake", c.connect)
 
+    def test_showssl(self):
+        assert not "certificate chain" in self.tval(["get:/p/200"], showssl=True)
+
     def test_ignorecodes(self):
         assert "200" in self.tval(["get:'/p/200:b@1'"])
         assert "200" not in self.tval(["get:'/p/200:b@1'"], ignorecodes=[200])
@@ -103,7 +130,7 @@ class TestDaemon(_TestDaemon):
         assert not "HTTP" in self.tval(["get:'/p/200:p5,10'"], showresp=True, timeout=0.01, ignoretimeout=True)
 
     def test_showresp(self):
-        reqs = [ "get:/api/info:p0,0", "get:/api/info:p0,0" ]
+        reqs = ["get:/api/info:p0,0", "get:/api/info:p0,0"]
         assert self.tval(reqs).count("200") == 2
         assert self.tval(reqs, showresp=True).count("unprintables escaped") == 2
         assert self.tval(reqs, showresp=True, hexdump=True).count("hex dump") == 2
@@ -114,28 +141,20 @@ class TestDaemon(_TestDaemon):
         assert "HTTP/" in v
 
     def test_explain(self):
-        reqs = [ "get:/p/200:b@100" ]
-        assert not "b@100" in self.tval(reqs, explain=True)
+        reqs = ["get:/p/200:b@100"]
+        assert "b@100" not in self.tval(reqs, explain=True)
 
     def test_showreq(self):
-        reqs = [ "get:/api/info:p0,0", "get:/api/info:p0,0" ]
+        reqs = ["get:/api/info:p0,0", "get:/api/info:p0,0"]
         assert self.tval(reqs, showreq=True).count("unprintables escaped") == 2
         assert self.tval(reqs, showreq=True, hexdump=True).count("hex dump") == 2
 
-    def test_parse_err(self):
-        assert "Error parsing" in self.tval(["foo"])
-
     def test_conn_err(self):
         assert "Invalid server response" in self.tval(["get:'/p/200:d2'"])
 
-    def test_fileread(self):
-        d = tutils.test_data.path("data/request")
-        assert "foo" in self.tval(["+%s"%d], showreq=True)
-        assert "File" in self.tval(["+/nonexistent"])
-
     def test_connect_fail(self):
         to = ("foobar", 80)
-        c = pathoc.Pathoc("127.0.0.1", self.d.port)
+        c = pathoc.Pathoc(("127.0.0.1", self.d.port))
         c.rfile, c.wfile = cStringIO.StringIO(), cStringIO.StringIO()
         tutils.raises("connect failed", c.http_connect, to)
         c.rfile = cStringIO.StringIO(
@@ -146,6 +165,3 @@ class TestDaemon(_TestDaemon):
             "HTTP/1.1 200 OK\r\n"
         )
         c.http_connect(to)
-
-
-
